### This is plain Perl code                       -*- mode: perl -*-

### Dear user,
###
### this configuration is admittedly very hackish, and should be simplified.
### No doubt I will do it some day, but it's not on top of my todo list.
### Feel free to improve it, and send me your patches back!
###
### Thanks, DGMS.

# Our address: this should ultimately be $ENV{'listreq'}.
$::from = "The administrator of $ENV{'list'} <$ENV{'listreq'}>";

# List of admissible MIME types -- beside "multipart/*" and "message/*", which
# both act as containers (don't ever even think to list "multipart/something"
# and/or "message/something" here: that would be a VERY BAD idea).
#
# Each item in the list may be a regexp.  The regexps are applied anchored with
# "^" and "$" added in front and back, so they should match the entire MIME
# type token, not just some substring.
#
# Parts of a type listed here will be accepted; you probably want to have
# at least '/text/plain', as a minimum.
@::ammessi = (
	      'text/plain',
	      'application/pgp-signature',
	      # 'foo/.*',
	      # '.*bar/foo',
	      );

# List of inadmissibile MIME types.
#
# Each item in the list may be a regexp.  The regexps are applied anchored with
# "^" and "$" added in front and back, so they should match the entire MIME
# type token, not just some substring.
#
# Parts of a type listed here will never be accepted, even if they match some
# regexp listed in @::ammessi.
#
# This list may be used to easily complement @::ammessi, but you should bear in
# mind that if a MIME type doesn't match some regexp in @::ammessi it won't be
# accepted in any way, even if it isn't listed here.  To be more clear, you do
# not need to list here every conceivable MIME type that you don't want in your
# messages, rather just the ones (if any) that would be otherwise accepted by
# some item in your @::ammessi list.
@::nonammessi = (
		 );

### The following here documents get included in our messages if the conditions
### hereby listed happen.

# TO.......: Author of the original message, as seen from (in decreasing order
#            of priority) Reply-To, From or Return-Path.
# CONDITION: MIME parsing of the original message has failed; this is very
#            very unlikely, but possible nevertheless.
@::invalido = <<'EOF;';
Hi,

this is an automatic reply.  Your message to this list

			HAS NOT BEEN DELIVERED,

because it was not a valid MIME message.


This probably means that your MUA had some technical problem.


If you find something unclear about this warning, feel free to reply to this
message.
EOF;

# TO.......: Author of the original message, as seen from (in decreasing order
#            of priority) Reply-To, From or Return-Path.
# CONDITION: Filtering resulted in an empty message (i.e., it was empty, or it
#            has become empty after filtering away unaccettable MIME parts).
@::vuoto = <<'EOF;';
Hi,

this is an automatic reply.  Your message to this list

			HAS NOT BEEN DELIVERED,

because it was empty after being processed to strip (any) non textual part.


Most often, this may be due to the following conditions:

  * You really sent an empty message: you may not send empty message to this
    list (a message is empty if its body contains no text).

  * Your MUA send messages in HTML only mode: you have to change your MUA
    configurations, we do not welcome HTML on this list.

  * You meant to send an attachment, such as a program, an image or a file: you
    may not send anything of this sort to this list.  If you have something you
    think worthwile, please, get in contact with the moderator of this list
    (you may reply to this message, for example), and s/he will tell you what
    to do.


If you find something unclear about this warning, feel free to reply to this
message.
EOF;

# TO.......: Author of the original message, as seen from (in decreasing order
#            of priority) Reply-To, From or Return-Path.
# CONDITION: Some MIME parts have been stripped from the original message
#            before letting it through.
@::mutilato = <<'EOF;';
Hi,

this is an automatic reply.  Your message to this list

		     HAS BEEN REGULARLY DELIVERED.

However, some non textual MIME parts have been stripped from it.

		YOU DO NOT NEED TO RESEND THIS MESSAGE.


Most often, this may be due to the following conditions:

  * Your MUA send messages both in text and HTML mode: you have to select text
    only mode for your messages to this list.

  * You meant to send an attachment, such as a program, an image or a file: you
    may not send anything of this sort to this list.  If you have something you
    think worthwile, please, get in contact with the moderator of this list
    (you may reply to this message, for example), and s/he will tell you what
    to do.


If you find something unclear about this warning, feel free to reply to this
message.
EOF;

# TO.......: Author of the original message, as seen from (in decreasing order
#            of priority) Reply-To, From or Return-Path.
# CONDITION: Added to the bottom of our message to the author if there has
#            been any stripped part.  A complete list of every stripped part
#            will be added to this message.
@::tagliato = <<'EOF;';

This is a list of the MIME types of every stripped part:
EOF;

### When we strip some part, we replace it with a short text explaining what
### was there.

# MIME descriptions of part and message replacements: think of it as a title.
$::descrizione = 'AUTOMATIC WARNING';

# This text replaces a stripped part (W: _TIPOMIME_ is a KEYWORD).
@::sostituto = <<"EOF;";
Here was a non admissible message part of \'_TIPOMIME_\' MIME type.
It has been automatically discarded before sending the message to the list.
EOF;

# This text replaces a stripped message (W: _TIPOMIME_ is a KEYWORD).
# Embedded messages are discarded if, and only if, they are or become empty.
# If the top most message entity becomes empty, nothing will get through.
@::messaggio = <<"EOF;";
Here was an embedded message of \'_TIPOMIME_\' MIME type.
It has been automatically discarded before sending the whole message to the
list, since it didn\'t contain any(more) textual part.
EOF;
